unit ApplSettings ;  { v.2.1 }

{ (c) Tom Peiffer 2000 }
{ For detailed description see readme.htm }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ntreg, inifiles, TypInfo ;

type
  TParamLocation = (Global, User);
  TPropFieldType = (pfBoolean,pfFloat,pfInteger,pfString) ;
  TSaveLocation = (Inifile,Registry) ;

  TApplParams = class ;
  TApplValues = class ;

  TApplValue = class (TCollectionItem)
  private
    FPropertyName: string;
    FKeyName: string;
    FDefaultValue: variant;
    FSection: string;
    procedure SetDefaultValue(const Value: variant);
    procedure SetKeyName(const Value: string);
    procedure SetPropertyName(const Value: string);
    procedure SetSection(const Value: string);
  protected
    procedure DefineProperties (Filer : TFiler) ; override ;
    function GetDisplayName : string ; override ;
  public
    constructor Create (Collection : TCollection) ; override ;
    destructor Destroy ; override ;
    procedure Assign (Source : TPersistent) ; override ;
  published
    property PropertyName : string  read FPropertyName write SetPropertyName;
    property KeyName : string  read FKeyName write SetKeyName;
    property Default : variant  read FDefaultValue write SetDefaultValue;
    property Section : string  read FSection write SetSection;
  end ;

  TApplCollection = class (TCollection)
  private
    FOwner : TApplParams ;
    function GetItem(Index: integer): TApplValue;
    procedure SetItem(Index: integer; const Value: TApplValue);
  protected
    function GetOwner : TPersistent ; override ;
    procedure Update (Item : TCollectionItem) ; override ;
  public
    constructor Create (AOwner : TApplParams) ;
    function Add : TApplValue ;
    property ApplParams : TApplParams read FOwner ;
    property Items[Index : integer]: TApplValue read GetItem write SetItem ; default ;
  end ;

  TApplParams = class(TComponent)
  private
    FUserName: string;
    FLocation: TParamLocation;
    procedure SetUserName(const Value: string);
    function GetUserName: string;
    function GetPropertyType(PInfo: PPropInfo): TPropFieldType;
    procedure SetLocation(const Value: TParamLocation);
    function MkDirEx(bez: string): boolean;
  private
    FOwner : TComponent ;
    FSaveOnExit: boolean;
    FLoadOnCreate: boolean;
    FValues: TApplCollection;
    FStorage: TApplValues;
    FSaveTo: TSaveLocation;
    FInifileDirectory: String;
    FRegistryMainKey: string;
    property UserName : string  read GetUserName write SetUserName;
    procedure SetLoadOnCreate(const Value: boolean);
    procedure SetSaveOnExit(const Value: boolean);
    procedure SetStorage(const Value: TApplValues);
    procedure SetValues(const Value: TApplCollection);
    procedure SetSaveTo(const Value: TSaveLocation);
    function LoadRegValue (reg : TNTRegistry; V : TApplValue) : boolean ;
    function SaveRegValue (reg : TNTRegistry ; V : TApplValue) : boolean ;
    function GetRegistryPath(Path, Section: string): string;
    function SaveIniValue(ini: TIniFile; V: TApplValue): boolean;
    function GetIniFileName(Path, Name: string): string;
    function LoadIniValue(ini: TIniFile; V: TApplValue): boolean;
    procedure SetInifileDirectory(const Value: String);
    procedure SetRegistryMainKey(const Value: string);
    { Private-Deklarationen}
  protected
    { Protected-Deklarationen}
  public
    { Public-Deklarationen}
    property Storage : TApplValues  read FStorage write SetStorage;
    constructor Create (AOwner : TComponent) ; override ;
    procedure Load ;
    procedure Save ;
  published
    { Published-Deklarationen }
    property LoadOnCreate : boolean  read FLoadOnCreate write SetLoadOnCreate;
    property SaveOnExit : boolean  read FSaveOnExit write SetSaveOnExit;
    property InifileDirectory : String  read FInifileDirectory write SetInifileDirectory;
    property RegistryMainKey : string  read FRegistryMainKey write SetRegistryMainKey;
    property Values : TApplCollection  read FValues write SetValues;
    property SaveTo : TSaveLocation  read FSaveTo write SetSaveTo;
    property Location : TParamLocation  read FLocation write SetLocation;
    { Events }
  end;


  { Von dieser Klasse aus muss eigene ApplValues Klasse abgeleitet werden }
  { welche die Schlsselwerte enthlt }
  TApplValues = class (TComponent)
  private
    FApplParams: TApplParams;
    procedure SetApplParams(const Value: TApplParams);
  public
    property ApplParams : TApplParams  read FApplParams write SetApplParams;
    destructor Destroy ; override ;
    procedure Load ;
    procedure Save ;
    constructor Create (AOwner : TComponent; ApplParams: TApplParams) ;
  end ;


procedure Register;

implementation
uses DsgnIntf, filectrl ;

const
 EmptySection = 'General' ;

procedure Register;
begin
  RegisterComponents('Tom', [TApplParams]);
end;


constructor TApplParams.Create(AOwner : TComponent);
begin
     inherited create (AOwner) ;
     FOwner := AOwner ;
     FLoadOnCreate := true ;
     FSaveOnExit := true ;
     FValues := TApplCollection.create (Self) ;
     FLocation := User ;
     FIniFileDirectory := '' ;
     FRegistryMainKey := '' ;
end;


constructor TApplValues.Create(AOwner : TComponent ; ApplParams: TApplParams);
begin
     inherited Create (AOwner) ;
     ApplParams.Storage := Self ;
     FApplParams := ApplParams ;

     if FApplParams.LoadOnCreate then FApplParams.Load ;
end;

destructor TApplValues.Destroy;
begin
     if FApplParams.SaveOnExit
     then FApplParams.Save ;
     inherited Destroy ;
end;

procedure TApplValues.Load;
begin
     FApplParams.Load ;
end;

procedure TApplValues.Save;
begin
     FApplParams.Save ;
end;


procedure TApplValue.Assign(Source: TPersistent);
begin
     inherited Assign (Source) ;
end;

constructor TApplValue.Create(Collection: TCollection);
begin
     inherited Create (Collection) ;
     FPropertyName := '' ;
end;

procedure TApplValue.DefineProperties(Filer: TFiler);
begin
     inherited DefineProperties (Filer) ;
end;

destructor TApplValue.Destroy;
begin
     inherited Destroy ;
end;

function TApplValue.GetDisplayName: string;
begin
     if (FPropertyName = '') and (FKeyName = '')
     then GetDisplayName := inherited GetDisplayName
     else GetDisplayName := FPropertyName + ' [' + FKeyName + ']' ;
end;

procedure TApplValue.SetDefaultValue(const Value: variant);
begin
  FDefaultValue := Value;
end;

procedure TApplValue.SetKeyName(const Value: string);
begin
  FKeyName := Value;
end;


procedure TApplValue.SetPropertyName(const Value: string);
begin
  FPropertyName := Value;
end;

function TApplCollection.Add: TApplValue;
begin
     Result := TApplValue(inherited Add) ;
end;

constructor TApplCollection.Create(AOwner: TApplParams);
begin
     inherited Create (TApplValue) ;
     FOwner := AOwner ;
end;

function TApplCollection.GetItem(Index: integer): TApplValue;
begin
     Result := TApplValue(inherited GetItem(index)) ;
end;

function TApplCollection.GetOwner: TPersistent;
begin
     Result := FOwner ;
end;

procedure TApplCollection.SetItem(Index: integer;
  const Value: TApplValue);
begin
     inherited SetItem (Index,Value) ;
end;

procedure TApplCollection.Update(Item: TCollectionItem);
begin
     //
end;


procedure TApplValue.SetSection(const Value: string);
begin
  FSection := Value;
end;

procedure TApplParams.Save;
var
 reg : TNTRegistry ;
 ini : TIniFile ;
 V : TApplValue ;
 i : integer ;
 iniFileName : string ;
begin
     for i := 0 to values.count-1 do begin
        V := values[i] ;

        { Save to REGISTRY }
        if FSaveTo = Registry then begin
            reg := TNTRegistry.create ;
            if FLocation = User then reg.RootKey := HKEY_CURRENT_USER ;
            if FLocation = Global then reg.RootKey := HKEY_LOCAL_MACHINE ;
            SaveRegValue (reg,V) ;
            reg.free ;
        end ;

        { Save to INIFILE }
        if FSaveTo = Inifile then begin
            if FLocation = User then iniFileName := UserName ;
            if FLocation = Global then iniFileName := 'Global' ;
            iniFileName := GetIniFileName (FIniFileDirectory,iniFileName) ;
            ini := TINIFile.create (INIFileName) ;
            SaveIniValue (ini,V) ;
            ini.free ;
        end ;

     end ;
end;

function TApplParams.GetPropertyType (PInfo : PPropInfo) : TPropFieldType ;
begin
     case PInfo^.PropType^.Kind of
       tkString, tkLString, tkWString :
              Result := pfString ;
       tkInteger:
              Result := pfInteger ;
       tkFloat:
              Result := pfFloat ;
       tkEnumeration:
              Result := pfBoolean ;
       else
              Result := pfString ;
     end ;
end ;

function TApplParams.SaveIniValue (ini : TIniFile ; V : TApplValue): boolean ;
var
 PInfo : PPropInfo ;
 AFloat : extended ;
 AInteger : integer ;
 sec, AString : string ;
 PT : TPropFieldType ;
begin
     PInfo := GetPropInfo (FStorage.ClassInfo,V.PropertyName) ;


     if PInfo <> nil then begin
         if V.Section <> ''
         then sec := V.Section
         else sec := EmptySection ;
         PT := GetPropertyType (PInfo) ;

         case PT of
              pfString : AString := GetStrProp (FStorage,Pinfo) ;
             pfInteger,
             pfBoolean : AInteger := GetOrdProp (FStorage,Pinfo) ;
               pfFloat : AFloat := GetFloatProp (FStorage,Pinfo) ;
         end ;

         case PT of
              pfString : ini.WriteString (Sec,V.KeyName,AString) ;
             pfInteger : ini.WriteInteger (Sec,V.KeyName,AInteger) ;
               pfFloat : ini.WriteFloat (Sec,V.KeyName,AFloat) ;
             pfBoolean : if abs(AInteger) = 1
                         then ini.WriteBool (Sec,V.KeyName,true)
                         else ini.WriteBool (Sec,V.KeyName,false) ;
         end ;
         Result := true ;
     end else begin
         Result := false ;
     end ;
end ;

function TApplParams.SaveRegValue(reg : TNTRegistry ; V: TApplValue): boolean;
var
 PInfo : PPropInfo ;
 AFloat : extended ;
 AInteger : integer ;
 AString : string ;
 PT : TPropFieldType ;
begin
     reg.OpenKey (GetRegistryPath(FRegistryMainKey,V.Section),true) ;
     PInfo := GetPropInfo (FStorage.ClassInfo,V.PropertyName) ;

     if PInfo <> nil then begin
         PT := GetPropertyType (PInfo) ;
         case PT of
              pfString : AString := GetStrProp (FStorage,Pinfo) ;
             pfInteger,
             pfBoolean : AInteger := GetOrdProp (FStorage,Pinfo) ;
               pfFloat : AFloat := GetFloatProp (FStorage,Pinfo) ;
         end ;

         case PT of
              pfString : reg.WriteString (V.KeyName,AString) ;
             pfInteger : reg.WriteInteger (V.KeyName,AInteger) ;
               pfFloat : reg.WriteFloat (V.KeyName,AFloat) ;
             pfBoolean : if abs(AInteger) = 1
                         then reg.WriteBool (V.KeyName,true)
                         else reg.WriteBool (V.KeyName,false) ;
         end ;
         Result := true ;
     end else begin
         Result := false ;
     end ;

     reg.CloseKey ;
end;


procedure TApplParams.Load;
var
 reg : TNTRegistry ;
 ini : TIniFile ;
 V : TApplValue ;
 i : integer ;
 IniFileName : string ;
begin
     for i := 0 to values.count-1 do begin
        V := values[i] ;

        { Load from REGISTRY }
        if FSaveTo = Registry then begin
            reg := TNTRegistry.create ;
            if FLocation = User then reg.RootKey := HKEY_CURRENT_USER ;
            if FLocation = Global then reg.RootKey := HKEY_LOCAL_MACHINE ;
            LoadRegValue (reg,V) ;
            reg.free ;
        end ;

        { Load from INIFILE }
        if FSaveTo = Inifile then begin
            if FLocation = User then iniFileName := UserName ;
            if FLocation = Global then iniFileName := 'Global' ;
            iniFileName := GetIniFileName (FIniFileDirectory,iniFileName) ;
            ini := TINIFile.create (INIFileName) ;
            LoadIniValue (ini,V) ;
            ini.free ;
        end ;

     end ;
end;

function TApplParams.GetRegistryPath (Path : string ; Section : string) : string ;
begin
     Result := Path ;
     if length(Path) > 0 then
       if Path[length(Path)] <> '\' then Result := Result + '\' ;

     if length(Section) > 0 then begin
         if Section[1] = '\' then Delete (Section,1,1) ;
     end ;

     Result := Result + Section ;
end ;

function TApplParams.GetIniFileName (Path : string ; Name : string) : string ;
begin
     if length(Path) > 0 then
     while Path[length(Path)] <> '\' do Path := Path + '\' ;

     if Path = ''
     then Path := ExtractFilePath(application.ExeName)
     else begin
            if not DirectoryExists (Path) then
            MkDirEx (Path) ;
          end ;

     Result := Path + Name + '.ini' ;
end ;

function TApplParams.MkDirEx (bez : string) : boolean ;
var
 verz : string ;
begin
     if copy (bez,length(bez),1) <> '\' then bez := bez + '\' ;
     result := false ;
     verz := copy (bez,1,2) ;
     delete (bez,1,3) ;
     while length(bez) > 0
     do begin
             verz := verz + '\' + copy (bez,1,pos('\',bez)-1) ;
             delete (bez,1,pos('\',bez)) ;
             {$I-}
                  mkdir (verz) ;
             {$I+}
             case IOResult of
                  0, 183 : begin end ;
                  else exit ;
             end ;
        end ;
     Result := true ;
end ;

function TApplParams.LoadRegValue(reg : TNTRegistry ; V: TApplValue): boolean;
var
 PInfo : PPropInfo ;
 AFloat : extended ;
 AInteger : integer ;
 AString : string ;
 PT : TPropFieldType ;
begin
     reg.OpenKey (GetRegistryPath(FRegistryMainKey,V.Section),true) ;
     PInfo := GetPropInfo (FStorage.ClassInfo,V.PropertyName) ;

     if PInfo <> nil then begin
         PT := GetPropertyType (PInfo) ;

         case PT of
           pfString : if reg.ValueExists (V.KeyName)
                      then AString := reg.ReadString (V.KeyName)
                      else AString := V.FDefaultvalue ;
           pfInteger,
           pfBoolean : if reg.ValueExists (V.KeyName)
                       then AInteger := reg.ReadInteger (V.KeyName)
                       else AInteger := V.FDefaultValue ;
           pfFloat : if reg.ValueExists (V.KeyName)
                     then AFloat := reg.ReadFloat (V.KeyName)
                     else AFloat := V.FDefaultValue ;
         end ;

         case PT of
            pfString : SetStrProp (FStorage,Pinfo,AString) ;
            pfInteger : SetOrdProp (FStorage,PInfo,AInteger) ;
            pfFloat : SetFloatProp (FStorage,Pinfo,AFloat) ;
            pfBoolean : if abs(AInteger) = 1
                        then SetOrdProp (FStorage,Pinfo,1)
                        else SetOrdProp (FStorage,Pinfo,0) ;
         end ;
         Result := true ;
     end else begin
         Result := false ;
     end ;

     reg.CloseKey ;
end;

function TApplParams.LoadIniValue(ini : TIniFile ; V: TApplValue): boolean;
var
 PInfo : PPropInfo ;
 AFloat : extended ;
 AInteger : integer ;
 sec, AString : string ;
 PT : TPropFieldType ;
begin
     PInfo := GetPropInfo (FStorage.ClassInfo,V.PropertyName) ;
     if PInfo <> nil then begin
         PT := GetPropertyType (PInfo) ;
         if V.Section <> ''
         then sec := V.Section
         else sec := EmptySection ;

         case PT of
           pfString : AString := ini.ReadString (Sec,V.KeyName,V.FDefaultValue) ;
           pfInteger,
           pfBoolean : AInteger := ini.ReadInteger (Sec,V.KeyName,V.FDefaultValue) ;
           pfFloat : AFloat := ini.ReadFloat (Sec,V.KeyName,V.FDefaultValue) ;
         end ;

         case PT of
            pfString : SetStrProp (FStorage,Pinfo,AString) ;
            pfInteger : SetOrdProp (FStorage,PInfo,AInteger) ;
            pfFloat : SetFloatProp (FStorage,Pinfo,AFloat) ;
            pfBoolean : if abs(AInteger) = 1
                        then SetOrdProp (FStorage,Pinfo,1)
                        else SetOrdProp (FStorage,Pinfo,0) ;
         end ;
         Result := true ;
     end else begin
         Result := false ;
     end ;
end;


procedure TApplParams.SetLoadOnCreate(const Value: boolean);
begin
  FLoadOnCreate := Value;
end;


procedure TApplValues.SetApplParams(const Value: TApplParams);
begin
  FApplParams := Value;
end;

procedure TApplParams.SetSaveOnExit(const Value: boolean);
begin
  FSaveOnExit := Value;
end;

procedure TApplParams.SetSaveTo(const Value: TSaveLocation);
begin
  FSaveTo := Value;
end;

procedure TApplParams.SetStorage(const Value: TApplValues);
begin
  FStorage := Value;
end;

procedure TApplParams.SetValues(const Value: TApplCollection);
begin
  FValues := Value;
end;

procedure TApplParams.SetInifileDirectory(const Value: String);
begin
  FInifileDirectory := Value;
end;

procedure TApplParams.SetRegistryMainKey(const Value: string);
begin
  FRegistryMainKey := Value;
end;

procedure TApplParams.SetUserName(const Value: string);
begin
  FUserName := Value;
end;

function TApplParams.GetUserName : string ;
var
   pcUser   : PChar;
   dwUSize : DWORD;
begin
   dwUSize := 21; // user name can be up to 20 characters
   GetMem( pcUser, dwUSize ); // allocate memory for the string
   try
      if Windows.GetUserName( pcUser, dwUSize )
      then begin
                Result := pcUser ;
                FUserName := Result ;
           end
      else begin
                Result := '' ;
                FUserName := '' ;
           end ;
   finally
      FreeMem( pcUser ); // now free the memory allocated for the string
   end;
end;

procedure TApplParams.SetLocation(const Value: TParamLocation);
begin
  FLocation := Value;
end;


end.
